function getCoordinates(startArc: number, endArc: number) {
  const posi = [
    Math.sin(startArc),
    -Math.cos(startArc),
    Math.sin(endArc),
    -Math.cos(endArc),
  ];
  const dx = posi[2] - posi[0];
  const dy = posi[3] - posi[1];

  return getLocation(dx, dy);
}
function getLocation(dx: number, dy: number) {
  const tanV = dx / dy;
  const directSign = Math.abs(tanV) < 1;
  const t = directSign ? tanV : 1 / tanV;

  const sign1 = t > 0 ? 1 : -1;
  const sign2 = dx > 0 ? 1 : -1;
  const sign = directSign ? sign1 * sign2 : sign2;

  const group1 = [0.5 - (sign * t) / 2, 0.5 + (sign * t) / 2];
  const group2 = sign > 0 ? [0, 1] : [1, 0];
  const group = [...group1, ...group2];
  const keys = directSign ? ["x", "x2", "y", "y2"] : ["y", "y2", "x", "x2"];

  let res: { [key: string]: any } = {};
  keys.forEach((k, idx) => {
    res[k] = group[idx];
  });
  return res;
}
let rangArrValue: any[] = [];
let dataList: any = [];
let totalValue = 0;
const colors = [
  "rgb(39, 96, 255)",
  "rgb(91, 155, 255)",
  "rgb(255, 209, 96)",
  "rgb(129, 103, 246)",
  "rgb(153, 214, 108)",
  "rgb(255, 138, 64)",
  "rgb(18, 255, 245)",
];
const getPersonnelList = async (dataProps: any) => {
  totalValue = dataProps.reduce(
    (total: any, value: any) => total + value.value,
    0
  );
  let cacheNum = 0;
  for (let i = 0; i < dataProps.length; i++) {
    const endNum = cacheNum + dataProps[i].value;
    rangArrValue.push([cacheNum, endNum]);
    cacheNum = endNum;
  }
  const angleArr = rangArrValue.map((arr) =>
    arr.map((num: any) => (num / totalValue) * Math.PI * 2)
  );
  dataList = dataProps.map((item: any, index: any) => {
    const range = getCoordinates(angleArr[index][0], angleArr[index][1]);
    const startColor = colors[index];
    const color = {
      type: "linear",
      x: range.x,
      x2: range.x2,
      y: range.y,
      y2: range.y2,
      colorStops: [
        {
          offset: 0,
          color: startColor, // 起始颜色
        },
        {
          offset: 1,
          color: `${startColor.substring(0, startColor.length - 1)}, 0)`, // 终点颜色
        },
      ],
      global: false,
    };

    return {
      name: item.name,
      value: item.value,
      itemStyle: {
        color: color,
      },
    };
  });
};
export default function getOptions(dataProps: any) {
  if (dataProps.length === 0) return null;
  getPersonnelList(dataProps);
  const colors = [
    "rgb(39, 96, 255)",
    "rgb(91, 155, 255)",
    "rgb(255, 209, 96)",
    "rgb(129, 103, 246)",
    "rgb(153, 214, 108)",
    "rgb(255, 138, 64)",
    "rgb(18, 255, 245)",
  ];
  return {
    tooltip: {
      trigger: "item",
      className: "handan-chart-tooltip",
    },
    graphic: [
      {
        type: "text",
        left: "center",
        top: "44%",
        style: {
          text: totalValue,
          fill: "#fff",
          width: 150,
          height: 48,
          fontSize: 31,
          fontWeight: 400,
        },
      },
      {
        type: "text",
        left: "center",
        top: "60%",
        style: {
          text: "总数",
          fill: "rgba(255, 255, 255, 0.70)",
          width: 32,
          height: 16,
          fontSize: 16,
          fontWeight: 400,
        },
      },
    ],
    series: [
      {
        name: "产线缺陷分类",
        type: "pie",
        radius: ["65%", "85%"],
        center: ["50%", "50%"],
        label: {
          formatter: "{d}%",
          color: "#fff",
          // color: function (params: any) {
          //   // 根据数据项的索引从颜色数组中选取颜色作为标签颜色
          //   var colorIndex = params.dataIndex;
          //   return colors[colorIndex];
          // },
        },
        data: dataList,
      },
    ],
  };
}
