import * as echarts from "echarts";
export default function getOptions(xData: any, outputList: any, rateList: any) {
  if (xData.length === 0 || outputList.length === 0 || rateList.length === 0)
    return null;
  return {
    grid: { top: 30, right: 48, bottom: 26, left: 48 },
    legend: {
      show: false,
    },
    xAxis: {
      type: "category",
      data: xData,
      axisLabel: {
        color: "rgba(223, 241, 254, 0.8)",
        fontSize: 12,
        interval: 0,
      },
      axisTick: { show: false },
      axisLine: {
        lineStyle: {
          width: 2,
          color: "rgba(69, 97, 174, 1)",
        },
      },
    },
    yAxis: [
      {
        name: "单位/片",
        nameTextStyle: {
          color: "rgba(223, 241, 254, 0.8)",
          fontSize: 12,
          align: "right",
        },
        type: "value",
        alignTicks: true,
        axisLabel: {
          color: "rgba(223, 241, 254, 0.65)",
          fontSize: 12,
          formatter: "{value}",
        },
        axisLine: {
          show: true,
          lineStyle: {
            width: 2,
            color: "rgba(69, 97, 174, 1)",
          },
        },
        splitLine: {
          lineStyle: {
            width: 1,
            color: "rgba(69, 97, 174, 1)",
          },
        },
      },
      {
        name: "单位/%",
        nameTextStyle: {
          color: "rgba(223, 241, 254, 0.8)",
          fontSize: 12,
          align: "center",
        },
        type: "value",
        axisLabel: {
          color: "rgba(223, 241, 254, 0.65)",
          fontSize: 12,
          formatter: "{value}",
        },
        axisLine: {
          show: true,
          lineStyle: {
            width: 2,
            color: "rgba(69, 97, 174, 1)",
          },
        },
        splitLine: {
          lineStyle: {
            width: 1,
            color: "rgba(69, 97, 174, 1)",
          },
        },
      },
    ],
    tooltip: {
      trigger: "axis",
      axisPointer: {
        type: "shadow",
      },
      className: "handan-chart-tooltip",
    },
    series: [
      {
        data: outputList,
        type: "bar",
        barWidth: 12,
        itemStyle: {
          color: new echarts.graphic.LinearGradient(0, 0, 0, 1, [
            { offset: 0, color: "#34C1FF" },
            { offset: 1, color: "#3449FF" },
          ]),
        },
      },
      {
        data: rateList,
        color: "rgba(18, 255, 245, 1)",
        type: "line",
        yAxisIndex: 1,
        symbol: "circle",
        symbolSize: 6,
        areaStyle: {
          color: new echarts.graphic.LinearGradient(0, 0, 0, 1, [
            { offset: 0, color: "rgba(18, 255, 245, 0.6)" },
            { offset: 0.4, color: "rgba(18, 255, 245, 0.1)" },
            { offset: 0.8, color: "rgba(18, 255, 245, 0)" },
            { offset: 1, color: "rgba(18, 255, 245, 0)" },
          ]),
        },
      },
    ],
  };
}
