import { createSlice } from "@reduxjs/toolkit";
import type { RootState } from "./store";

export interface Alarm {
  alarmContent: string;
  equName: string;
  alarmCode: any;
  alarmEquipmentId: number;
  alarmValue: string;
  alarmId: number;
  alarmSource: string;
}

export interface TickCount {
  equName: string;
  tickCount: number;
}

export interface Product {
  externalCode: number;
  equipmentCode: number;
  outputNum: number;
  lineName: string;
  equipmentId: number;
  sectionName: string;
  inputNum: number;
  recordTime: number;
  equipmentName: string;
}

export interface Oee {
  workRate: number;
  equName: string;
  downRate: number;
  stopRate: number;
}

export interface EquipmentLine {
  equipmentAlarm: Array<Alarm>;
  equipmentTickCounts: Array<TickCount>;
  equipmentOees: Array<Oee>;
  equipmentProductDays: Array<Product>;
  equipmentProductWeeks: Array<Product>;
  equipmentProductMonths: Array<Product>;
  equipmentProductAll: Array<Product>;
}

export interface EquipmentMonitorEntityInterface {
  Line_1: EquipmentLine;
  Line_2: EquipmentLine;
  Line_3: EquipmentLine;
  Line_4: EquipmentLine;
}

const emptyEquipmentLine: EquipmentLine = {
  equipmentAlarm: [],
  equipmentTickCounts: [],
  equipmentOees: [],
  equipmentProductDays: [],
  equipmentProductWeeks: [],
  equipmentProductMonths: [],
  equipmentProductAll: [],
};

const initialState: EquipmentMonitorEntityInterface = {
  Line_1: emptyEquipmentLine,
  Line_2: emptyEquipmentLine,
  Line_3: emptyEquipmentLine,
  Line_4: emptyEquipmentLine,
};

export const EquipmentMonitorEntity = createSlice({
  name: "EquipmentMonitorEntity",
  initialState,
  reducers: {
    UpdateEquipmentMonitorEntity: (state, action) => {
      if (action.payload.toString().includes("客户端")) {
      } else {
        const BasicEquipmentMonitorEntity = JSON.parse(action.payload);
        state.Line_1 = BasicEquipmentMonitorEntity.Line_1;
        state.Line_2 = BasicEquipmentMonitorEntity.Line_2;
        state.Line_3 = BasicEquipmentMonitorEntity.Line_3;
        state.Line_4 = BasicEquipmentMonitorEntity.Line_4;
      }
    },
  },
});

export const { UpdateEquipmentMonitorEntity } = EquipmentMonitorEntity.actions;

// export const selectAllLineEquipmentData = (state: RootState) => state.EquipmentMonitorEntity;

export default EquipmentMonitorEntity.reducer;
