import {createSlice} from "@reduxjs/toolkit";
import type {RootState} from "./store";

export interface alarm {
  alarmContent: string;
  alarmTime: string;
  alarmCode: string;
}

export interface ProductionDet {
  lineName: string;
  inputNum: number;
  outputNum: number;
  passRate: number;
}

export interface ProductionRate {
  lineName: string;
  lineId: number;
  lineExCode: string;
  passRate: number;
  time: number;
}

export interface RateLine {
  Line_1: Array<ProductionRate>,
  Line_2: Array<ProductionRate>,
  Line_3: Array<ProductionRate>,
  Line_4: Array<ProductionRate>,
}

export interface SingleGlassStatus {
  lineViewCode: string,
  status: number
}

export interface GlassStatus {
  LINE_1_1: number,
  LINE_1_2U: number,
  LINE_1_2D: number,
  LINE_1_3: number,
  LINE_1_4: number,
  LINE_2_1: number,
  LINE_2_2U: number,
  LINE_2_2D: number,
  LINE_2_3: number,
  LINE_2_4: number,
  LINE_3_1: number,
  LINE_3_2U: number,
  LINE_3_2D: number,
  LINE_3_3: number,
  LINE_3_4: number,
  LINE_4_1: number,
  LINE_4_2U: number,
  LINE_4_2D: number,
  LINE_4_3: number,
  LINE_4_4: number,
}

const initLineGlassStatus: GlassStatus = {
  LINE_1_1: 1,
  LINE_1_2U: 1,
  LINE_1_2D: 1,
  LINE_1_3: 1,
  LINE_1_4: 1,
  LINE_2_1: 1,
  LINE_2_2U: 1,
  LINE_2_2D: 1,
  LINE_2_3: 1,
  LINE_2_4: 1,
  LINE_3_1: 1,
  LINE_3_2U: 1,
  LINE_3_2D: 1,
  LINE_3_3: 1,
  LINE_3_4: 1,
  LINE_4_1: 1,
  LINE_4_2U: 1,
  LINE_4_2D: 1,
  LINE_4_3: 1,
  LINE_4_4: 1,
}

export interface ProductionMonitoringEntityInterface {
  sumNumber: number;
  alarms: Array<alarm>;
  sumProductionDets: Array<ProductionDet>;
  todayProductionDets: Array<ProductionDet>;
  weekProductionDets: Array<ProductionDet>;
  monthProductionDets: Array<ProductionDet>;
  todayProductionRates: RateLine,
  weekProductionRates: RateLine,
  monthProductionRates: RateLine,
  lineGlassStatus: GlassStatus,
}

const emptyProductionRates: RateLine = {
  Line_4: [
    {
      "lineId": 1,
      "lineName": "产线1",
      "lineExCode": "Line_1",
      "passRate": 101.88,
      "time": 1676840400000
    },
  ],
  Line_3: [
    {
      "lineId": 1619974755856867329,
      "lineName": "产线3",
      "lineExCode": "Line_3",
      "passRate": 32.04,
      "time": 1676822400000
    }
  ],
  Line_2: [
    {
      "lineId": 1619974755856867329,
      "lineName": "产线3",
      "lineExCode": "Line_3",
      "passRate": 32.04,
      "time": 1676822400000
    }
  ],
  Line_1: [
    {
      "lineId": 1619974755856867329,
      "lineName": "产线3",
      "lineExCode": "Line_3",
      "passRate": 32.04,
      "time": 1676822400000
    }
  ]
}

const initialState: ProductionMonitoringEntityInterface = {
  sumNumber: 0,
  alarms: [],
  sumProductionDets: [],
  todayProductionDets: [],
  weekProductionDets: [],
  monthProductionDets: [],
  todayProductionRates: emptyProductionRates,
  weekProductionRates: emptyProductionRates,
  monthProductionRates: emptyProductionRates,
  lineGlassStatus: initLineGlassStatus,
}

export const ProductionMonitoringEntity = createSlice({
  name: 'ProductionMonitoringEntity',
  initialState,
  reducers: {
    UpdateProductionMonitoringEntity: (state, action) => {
      if (action.payload.toString().includes('客户端')) {
      } else {
        const ProductionMonitoringEntityData = JSON.parse(action.payload);

        //sumNumber
        state.sumNumber = ProductionMonitoringEntityData.alarms.length;

        //alarms
        state.alarms = [];
        for (let i = 0; i < 6; i++) {
          state.alarms.push({
              alarmContent: ProductionMonitoringEntityData.alarms[i].alarmContent,
              alarmCode: ProductionMonitoringEntityData.alarms[i].alarmCode,
              alarmTime: new Date(ProductionMonitoringEntityData.alarms[i].alarmTime).toLocaleDateString(),
            }
          )
        }

        //sumProductionDets
        state.sumProductionDets = ProductionMonitoringEntityData.sumProductionDets;

        //todayProductionDets
        state.todayProductionDets = ProductionMonitoringEntityData.todayProductionDets;


        //weekProductionDets
        state.weekProductionDets = ProductionMonitoringEntityData.weekProductionDets;

        //monthProductionDets
        state.monthProductionDets = ProductionMonitoringEntityData.monthProductionDets;

        //todayProductionRates
        state.todayProductionRates = ProductionMonitoringEntityData.todayProductionRates;

        //weekProductionRates
        state.weekProductionRates = ProductionMonitoringEntityData.weekProductionRates;

        //monthProductionRates
        state.monthProductionRates = ProductionMonitoringEntityData.monthProductionRates;

      }
    },
    UpdateGlassStatus: (state, action) => {
      //lineGlassStatus
      state.lineGlassStatus = action.payload;
    }
  }
})

export const {UpdateProductionMonitoringEntity, UpdateGlassStatus} = ProductionMonitoringEntity.actions;

export const selectAlarms = (state: RootState) => state.ProductionMonitoringEntity.alarms;
export const selectSumNumber = (state: RootState) => state.ProductionMonitoringEntity.sumNumber;
export const selectSumProductionDets = (state: RootState) => state.ProductionMonitoringEntity.sumProductionDets;
export const selectTodayProductionDets = (state: RootState) => state.ProductionMonitoringEntity.todayProductionDets;
export const selectWeekProductionDets = (state: RootState) => state.ProductionMonitoringEntity.weekProductionDets;
export const selectMonthProductionDets = (state: RootState) => state.ProductionMonitoringEntity.monthProductionDets;
export const selectTodayProductionRates = (state: RootState) => state.ProductionMonitoringEntity.todayProductionRates;
export const selectWeekProductionRates = (state: RootState) => state.ProductionMonitoringEntity.weekProductionRates;
export const selectMonthProductionRates = (state: RootState) => state.ProductionMonitoringEntity.monthProductionRates;
export const selectGlassStatus = (state: RootState) => state.ProductionMonitoringEntity.lineGlassStatus;
export default ProductionMonitoringEntity.reducer;